@@ -191,7 +191,7 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
 }
 
 
-static typedef std::list< sharedptr<const UsesRelationship> > type_list_relationships;
+typedef std::list< sharedptr<const UsesRelationship> > type_list_relationships;
 
 static void add_to_relationships_list(type_list_relationships& list_relationships, const sharedptr<const LayoutItem_Field>& layout_item)
 {
@@ -327,6 +327,12 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
   if(!where_clause.empty())
     result += " WHERE " + where_clause;
 
+  //Extra GROUP_BY clause for doubly-related records. This must be before the ORDER BY sort clause:
+  if(!extra_group_by.empty())
+  {
+    result += (" " + extra_group_by + " ");
+  }
+
   //Sort clause:
   if(!sort_clause.empty())
   {
@@ -349,11 +355,6 @@ Glib::ustring Utils::build_sql_select_with_where_clause(const Glib::ustring& tab
       result += " ORDER BY " + str_sort_clause;
   }
 
-  if(!extra_group_by.empty())
-  {
-    result += (" " + extra_group_by + " ");
-  }
-
   return result;
 }
 
@@ -419,7 +420,7 @@ Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<con
 
 
   //std::cout << "get_choice_values: Executing SQL: " << sql_query << std::endl;
-  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = connection->get_gda_connection()->execute_single_command(sql_query);
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = connection->get_gda_connection()->execute_select_command(sql_query);
 
   if(datamodel)
   {
@@ -431,7 +432,7 @@ Utils::type_list_values_with_second Utils::get_choice_values(const sharedptr<con
       itempair.first = datamodel->get_value_at(0, row);
 
       if(with_second)
-			itempair.second = datamodel->get_value_at(1, row);
+                        itempair.second = datamodel->get_value_at(1, row);
 
       list_values.push_back(itempair);
     }
@@ -509,33 +510,24 @@ Glib::ustring Utils::create_local_image_uri(const Gnome::Gda::Value& value)
 
   Glib::ustring result;
 
-  if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_BINARY)
+  if(value.get_value_type() == GDA_TYPE_BINARY)
   {
     glong size = 0;
-    const gpointer pData = value.get_binary(size);
+    gconstpointer pData = value.get_binary(size);
     if(size && pData)
     {
-      //libgda does not currently properly unescape binary data,
-      //so pData is actually a null terminated string, of escaped binary data.
-      //This workaround should be removed when libgda is fixed:
-      //(It is fixed in libgd-2.0 but is unlikely to be fixed in libgda-1.2)
-      size_t buffer_binary_length = 0;
-      guchar* buffer_binary =  Glom_PQunescapeBytea((const guchar*)pData /* must be null-terminated */, &buffer_binary_length); //freed by us later.
-      if(buffer_binary)
+      // Note that this is regular binary data, not escaped text representing the data:
+
+      //Save the image to a temporary file and provide the file URI.
+      char pchExtraNum[10];
+      sprintf(pchExtraNum, "%d", m_temp_image_uri_number);
+      result = ("/tmp/glom_report_image_" + Glib::ustring(pchExtraNum) + ".png");
+      ++m_temp_image_uri_number;
+
+      std::fstream the_stream(result.c_str(), std::ios_base::out | std::ios_base::trunc | std::ios_base::binary);
+      if(the_stream)
       {
-        //Saves the image to a temporary file and provides the file URI.
-        char pchExtraNum[10];
-        sprintf(pchExtraNum, "%d", m_temp_image_uri_number);
-        result = ("/tmp/glom_report_image_" + Glib::ustring(pchExtraNum) + ".png");
-        ++m_temp_image_uri_number;
-
-        std::fstream the_stream(result.c_str(), std::ios_base::out | std::ios_base::trunc | std::ios_base::binary);
-        if(the_stream)
-        {
-          the_stream.write((char*)buffer_binary, buffer_binary_length);
-        }
-
-        free(buffer_binary);
+        the_stream.write((char*)pData, size);
       }
     }
     else
@@ -656,7 +648,7 @@ Utils::type_vecStrings Utils::string_separate(const Glib::ustring& str, const Gl
           //Which quote, if any, is first:
           Glib::ustring::size_type posFirstQuote = posSingleQuote;
           if( (posDoubleQuote != Glib::ustring::npos) && (posDoubleQuote < posFirstQuote) )
-	    posFirstQuote = posDoubleQuote;
+            posFirstQuote = posDoubleQuote;
 
           //Ignore quotes that are _after_ the separator:
           if( posFirstQuote >= posComma)
@@ -708,10 +700,16 @@ Utils::type_vecStrings Utils::string_separate(const Glib::ustring& str, const Gl
         //Store this item, and start the next item after it:
         item = str.substr(item_start, posComma - item_start);
         //std::cout << "  ITEM. pos_comma=" << posComma << ", ITEM= " << item << std::endl;
-	item_start = posComma + size_separator;
+        item_start = posComma + size_separator;
+      }
+      else
+      {
+        //std::cout << "in quotes." << std::endl;
+        // Continue behind separator
+        unprocessed_start = posComma + size_separator;
+        // Do not add this item to the result, because it was quoted.
+        continue;
       }
-      //else
-      //  std::cout << "in quotes." << std::endl;
      
       unprocessed_start = posComma + size_separator; //The while loops stops when this is empty.
     }
@@ -817,4 +815,13 @@ void Utils::show_help(const Glib::ustring& id)
    }
 }
 
+
+void Utils::show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message, Gtk::Window& parent, Gtk::MessageType message_type)
+{
+  Gtk::MessageDialog dialog("<b>" + title + "</b>", true /* markup */, message_type, Gtk::BUTTONS_OK);
+  dialog.set_secondary_text(message);
+  dialog.set_transient_for(parent);
+  dialog.run();
+}
+
 } //namespace Glom
